#include "double_audio_kernel.h"

#define MINIMUM_PARALLEL_ 100000

typedef struct {
  ulong nIn;
  ulong nOut;
  DoubleAudio *in;
  DoubleAudio *out;
} Info_;

static double read_(long in, void *x);

static void write_(ulong out, double v, void *x);


DoubleAudio *double_audio_new_apply_repeating_kernel(
    DoubleAudio **p_this,
    DoubleAudio *other,
    RepeatingKernel *kernel) {
  ulong rate_in;
  ulong rate_out;
  ulong kernel_n_in;
  ulong kernel_n_out;
  ulong n_in;
  ulong n_out;
  Info_ info;
  DoubleAudio *this;

  n_in = other->n;
  rate_in = other->rate;

  kernel_n_in = kernel->nIn;
  kernel_n_out = kernel->nOut;

  if (rate_in * kernel_n_out % kernel_n_in != 0) {
    throw("impossible");
  }

  rate_out = rate_in * kernel_n_out / kernel_n_in;
  n_out = (n_in * kernel_n_out + kernel_n_in / 2) / kernel_n_in;

  double_audio_new(p_this, n_out, rate_out);
  this = *p_this;

  info.nIn = n_in;
  info.nOut = n_out;
  info.in = other;
  info.out = this;

  repeating_kernel_apply(kernel, n_out, read_, write_, &info);

  return this;
}


static double read_(long in, void *x) {
  Info_ *info;

  info = x;

  if (in < 0) {
    return info->in->_samples[0];
  }

  if (in >= info->nIn) {
    return info->in->_samples[info->nIn - 1];
  }

  return info->in->_samples[in];
}

static void write_(ulong out, double v, void *x) {
  Info_ *info;

  info = x;

  if (out >= info->nOut) {
    throw("impossible %lu v %lu", out, info->nOut);
  }

  info->out->_samples[out] = v;
}
