#include "magic_double_audio.h"

#define A_ 6
#define V_ 7

DoubleAudio *double_audio_new_resample(
    DoubleAudio **p_this,
    DoubleAudio *other,
    ulong new_rate,
    bool extra_antialiasing) {
  ulong rate_in;
  ulong rate_out;
  RepeatingKernel *_sharpen = null;
  RepeatingKernel *_resample = null;
  DoubleAudio *_temp = null;

  rate_in = other->rate;
  rate_out = new_rate;

  if (rate_out == rate_in) {
    throw("Cannot resample to the same rate of %u Hz", rate_in);
  }

  if (rate_out > rate_in || !extra_antialiasing) {
    repeating_kernel_new_magic_sharpen(&_sharpen, A_, V_);
  }

  if (rate_out > rate_in) {
    double_audio_new_apply_repeating_kernel(&_temp, other, _sharpen);
    repeating_kernel_delete(&_sharpen);

    repeating_kernel_new_magic_kernel_resample(
        &_resample,
        rate_in,
        rate_out,
        A_);
    double_audio_new_apply_repeating_kernel(p_this, _temp, _resample);
    repeating_kernel_delete(&_resample);
    double_audio_delete(&_temp);

    return *p_this;
  }

  if (extra_antialiasing) {
    repeating_kernel_new_magic_kernel_sharp_safer_downsample(
        &_resample,
        rate_in,
        rate_out,
        A_,
        V_);
    double_audio_new_apply_repeating_kernel(p_this, other, _resample);
    repeating_kernel_delete(&_resample);

    return *p_this;
  }

  repeating_kernel_new_magic_kernel_resample(&_resample, rate_in, rate_out, A_);
  double_audio_new_apply_repeating_kernel(&_temp, other, _resample);
  repeating_kernel_delete(&_resample);

  double_audio_new_apply_repeating_kernel(p_this, _temp, _sharpen);
  repeating_kernel_delete(&_sharpen);
  double_audio_delete(&_temp);

  return *p_this;
}
