#include "wav_double_halve.h"
#include <math.h>

int main(int argc, char *argv[]) {
  bool halve = true;
  byte o;
  uint n_in;
  uint n_out;
  uint n_temp;
  uint rate_in;
  uint rate_out;
  uint u;
  uint v;
  double signal;
  double m1 = 31927.0 / 61440;
  double m2 = 6719.0 / 20480;
  double m3 = 15349.0 / 122880;
  double m4 = 3119.0 / 122880;
  double m5 = 81.0 / 40960;
  double m6 = 1.0 / 122880;
  double s0 = 8415120629.0 / 2952708336;
  double s1 = -3913360945.0 / 2952708336;
  double s2 = 20369223277.0 / 35432500032;
  double s3 = -4387620835.0 / 17716250016;
  double s4 = 1887357215.0 / 17716250016;
  double s5 = -808049411.0 / 17716250016;
  double s6 = 674477135.0 / 35432500032;
  double s7 = -3769012.0 / 553632813;
  double halver[12];
  double doubler[2][6];
  double sharp[15];
  double *in_samples;
  double *temp_samples;
  double *out_samples;
  DoubleAudio *_in = null;
  DoubleAudio *_temp = null;
  DoubleAudio *_out = null;

  halver[0] = halver[11] = m6 / 2;
  halver[1] = halver[10] = m5 / 2;
  halver[2] = halver[9] = m4 / 2;
  halver[3] = halver[8] = m3 / 2;
  halver[4] = halver[7] = m2 / 2;
  halver[5] = halver[6] = m1 / 2;

  doubler[0][0] = doubler[1][5] = m5;
  doubler[0][1] = doubler[1][4] = m3;
  doubler[0][2] = doubler[1][3] = m1;
  doubler[0][3] = doubler[1][2] = m2;
  doubler[0][4] = doubler[1][1] = m4;
  doubler[0][5] = doubler[1][0] = m6;

  sharp[0] = sharp[14] = s7;
  sharp[1] = sharp[13] = s6;
  sharp[2] = sharp[12] = s5;
  sharp[3] = sharp[11] = s4;
  sharp[4] = sharp[10] = s3;
  sharp[5] = sharp[9] = s2;
  sharp[6] = sharp[8] = s1;
  sharp[7] = s0;

  args_description(
      "Upsample or downsample a 16-bit PCM WAV file by a factor of 2");
  args_string("input-filename", "Input 16-bit PCM WAV file");
  args_string("output-filename", "Output WAV file to create");
  args_flag("halve", 'H', "Halve, i.e. downsample (e.g. from 96kHz to 48kHz)");
  args_flag("double", 'd', "Double, i.e. upsample (e.g. from 48kHz to 96kHz)");
  args_parse(argc, argv);

  if (args_set_veto("halve", "double")) {
    halve = true;
  } else if (args_set("double")) {
    halve = false;
  } else {
    throw("Must specify either --halve or --double");
  }

  double_audio_new_load_wav(&_in, args_get_string("input-filename"));
  n_in = _in->n;
  rate_in = _in->rate;
  in_samples = _in->_samples;

  if (halve) {
    if (rate_in < 96000) {
      throw("Program not intended to downsample to less than 48kHz");
    }

    n_temp = (n_in - 10) / 2;
    rate_out = rate_in / 2;
    double_audio_new(&_temp, n_temp, rate_out);
    temp_samples = _temp->_samples;

    for (u = 0; u < n_temp; u++) {
      for (signal = 0, v = 0; v < 12; v++) {
        signal += halver[v] * in_samples[2 * u + v];
      }

      temp_samples[u] = signal;
    }

    n_out = n_temp - 14;
    double_audio_new(&_out, n_out, rate_out);
    out_samples = _out->_samples;

    for (u = 0; u < n_out; u++) {
      for (signal = 0, v = 0; v < 15; v++) {
        signal += sharp[v] * temp_samples[u + v];
      }

      out_samples[u] = signal;
    }
  } else {
    n_temp = n_in - 14;
    double_audio_new(&_temp, n_temp, rate_in);
    temp_samples = _temp->_samples;

    for (u = 0; u < n_temp; u++) {
      for (signal = 0, v = 0; v < 15; v++) {
        signal += sharp[v] * in_samples[u + v];
      }

      temp_samples[u] = signal;
    }

    n_out = 2 * n_temp - 10;
    rate_out = rate_in * 2;
    double_audio_new(&_out, n_out, rate_out);
    out_samples = _out->_samples;

    for (u = 0; u < n_out;) {
      for (o = 0; o < 2; o++, u++) {
        for (signal = 0, v = 0; v < 6; v++) {
          signal += doubler[o][v] * temp_samples[u / 2 + v];
        }

        out_samples[u] = signal;
      }
    }
  }

  double_audio_save_wav(_out, args_get_string("output-filename"));

  double_audio_delete(&_out);
  double_audio_delete(&_temp);
  double_audio_delete(&_in);

  return 0;
}
